@@ -15,6 +15,9 @@ import (
 func (root *Root) toggleWrapMode() {
 	root.Doc.WrapMode = !root.Doc.WrapMode
 	root.Doc.x = 0
+	if root.Doc.ColumnMode {
+		root.Doc.x = root.columnModeX()
+	}
 	root.setMessagef("Set WrapMode %t", root.Doc.WrapMode)
 }
 
@@ -47,6 +50,11 @@ func (root *Root) toggleFollowAll() {
 	root.General.FollowAll = !root.General.FollowAll
 }
 
+// toggleFollowSection toggles follow section mode.
+func (root *Root) toggleFollowSection() {
+	root.Doc.FollowSection = !root.Doc.FollowSection
+}
+
 // closeFile close the file.
 func (root *Root) closeFile() {
 	if root.screenMode != Docs {
@@ -82,7 +90,11 @@ func (root *Root) reload(m *Document) {
 
 // toggleWatch toggles watch mode.
 func (root *Root) toggleWatch() {
-	root.Doc.WatchMode = !root.Doc.WatchMode
+	if root.Doc.WatchMode {
+		root.Doc.unWatchMode()
+	} else {
+		root.Doc.watchMode()
+	}
 	if root.Doc.WatchMode {
 		root.watchStart()
 	}
@@ -105,8 +117,7 @@ func (root *Root) watchStart() {
 				ev := &eventReload{}
 				ev.SetEventNow()
 				ev.m = m
-				err := root.Screen.PostEvent(ev)
-				if err != nil {
+				if err := root.Screen.PostEvent(ev); err != nil {
 					log.Println(err)
 				}
 			} else {
@@ -184,7 +195,7 @@ func (root *Root) markPrev() {
 
 // addMark marks the current line number.
 func (root *Root) addMark() {
-	c := min(root.Doc.topLN+root.Doc.firstLine(), root.Doc.endNum)
+	c := min(root.Doc.topLN+root.Doc.firstLine(), root.Doc.BufEndNum())
 	root.Doc.marked = removeInt(root.Doc.marked, c)
 	root.Doc.marked = append(root.Doc.marked, c)
 	root.setMessagef("Marked to line %d", c-root.Doc.firstLine()+1)
@@ -235,7 +246,7 @@ func (root *Root) setSkipLines(input string) {
 		root.setMessagef("Set skip line: %s", ErrInvalidNumber.Error())
 		return
 	}
-	if num < 0 || num > root.vHight-1 {
+	if num < 0 {
 		root.setMessagef("Set skip line: %s", ErrOutOfRange.Error())
 		return
 	}
@@ -303,7 +314,8 @@ func (root *Root) setViewMode(input string) {
 		c = root.General
 	}
 
-	root.Doc.general = c
+	root.Doc.general = overwriteGeneral(root.Doc.general, c)
+	root.Doc.setSectionDelimiter(root.Doc.SectionDelimiter)
 	root.Doc.ClearCache()
 	root.ViewSync()
 	root.setMessagef("Set mode %s", input)
@@ -331,6 +343,7 @@ func (root *Root) setTabWidth(input string) {
 	root.Doc.ClearCache()
 }
 
+// setWatchInterval sets the Watch interval.
 func (root *Root) setWatchInterval(input string) {
 	interval, err := strconv.Atoi(input)
 	if err != nil {
@@ -352,6 +365,8 @@ func (root *Root) setWatchInterval(input string) {
 	root.setMessagef("Set watch interval %d", interval)
 }
 
+// setWriteBA sets the number before and after the line
+// to be written at the end.
 func (root *Root) setWriteBA(input string) {
 	ba := strings.Split(input, ":")
 	bstr := ba[0]
@@ -382,6 +397,24 @@ func (root *Root) setWriteBA(input string) {
 	root.Quit()
 }
 
+// setSectionDelimiter sets the delimiter string.
+func (root *Root) setSectionDelimiter(input string) {
+	root.Doc.setSectionDelimiter(input)
+	root.setMessagef("Set section delimiter %s", input)
+}
+
+// setSectionStart sets the section start position.
+func (root *Root) setSectionStart(input string) {
+	num, err := strconv.Atoi(input)
+	if err != nil {
+		root.setMessagef("Set section start position: %s", ErrInvalidNumber.Error())
+		return
+	}
+
+	root.Doc.SectionStartPosition = num
+	root.setMessagef("Set section start position %s", input)
+}
+
 // resize is a wrapper function that calls viewSync.
 func (root *Root) resize() {
 	root.ViewSync()
@@ -401,6 +434,17 @@ func (root *Root) TailSync() {
 	root.ViewSync()
 }
 
+// tailSection moves to the last section
+// and adjusts to its original position.
+func (root *Root) tailSection() {
+	moved := root.Doc.topLN - root.Doc.lastSectionPosNum
+	root.lastSection()
+	if moved > 0 && (root.Doc.topLN+moved) < root.Doc.BufEndNum() {
+		root.moveLine(root.Doc.topLN + moved)
+	}
+	root.Doc.lastSectionPosNum = root.Doc.topLN
+}
+
 // prepareStartX prepares startX.
 func (root *Root) prepareStartX() {
 	root.startX = 0
